package ru.dbotthepony.kstarbound.lua

import org.classdump.luna.StateContext
import org.classdump.luna.Table
import org.classdump.luna.Variable
import org.classdump.luna.compiler.CompilerChunkLoader
import org.classdump.luna.compiler.CompilerSettings
import org.classdump.luna.env.RuntimeEnvironments
import org.classdump.luna.exec.DirectCallExecutor
import org.classdump.luna.impl.StateContexts
import org.classdump.luna.lib.BasicLib
import org.classdump.luna.lib.CoroutineLib
import org.classdump.luna.lib.MathLib
import org.classdump.luna.lib.OsLib
import org.classdump.luna.lib.StringLib
import org.classdump.luna.lib.TableLib
import org.classdump.luna.lib.Utf8Lib
import org.classdump.luna.runtime.Dispatch
import org.classdump.luna.runtime.ExecutionContext
import java.util.concurrent.atomic.AtomicInteger

class NewLuaState {
	val state: StateContext = StateContexts.newDefaultInstance()
	val chunkLoader: CompilerChunkLoader = CompilerChunkLoader.of(CompilerSettings.defaultNoAccountingSettings(), "kstarbound_${COUNTER.getAndIncrement()}_")
	val env: Table = state.newTable()
	val executor: DirectCallExecutor = DirectCallExecutor.newExecutor()

	init {
		env["_G"] = env

		env["assert"] = BasicLib.assertFn()
		env["error"] = BasicLib.error()
		env["getmetatable"] = BasicLib.getmetatable()
		env["ipairs"] = BasicLib.ipairs()
		env["next"] = BasicLib.next()
		env["pairs"] = BasicLib.pairs()
		env["pcall"] = BasicLib.pcall()

		env["rawequal"] = BasicLib.rawequal()
		env["rawget"] = BasicLib.rawget()
		env["rawlen"] = BasicLib.rawlen()
		env["rawset"] = BasicLib.rawset()
		env["select"] = BasicLib.select()
		env["setmetatable"] = BasicLib.setmetatable()
		env["tostring"] = BasicLib.tostring()
		env["tonumber"] = BasicLib.tonumber()
		env["type"] = BasicLib.type()
		env["_VERSION"] = BasicLib._VERSION
		env["xpcall"] = BasicLib.xpcall()

		env["print"] = PrintFunction(env)

		env["require"] = LuaRequire(this)

		CoroutineLib.installInto(state, env)
		TableLib.installInto(state, env)
		MathLib.installInto(state, env)
		StringLib.installInto(state, env)
		OsLib.installInto(state, env, RuntimeEnvironments.system())

		// TODO: NYI, maybe polyfill?
		Utf8Lib.installInto(state, env)
	}

	fun load(code: String, name: String = "main chunk") {
		val main = chunkLoader.loadTextChunk(Variable(env), name, code)
		executor.call(state, main)
	}

	fun load(context: ExecutionContext, code: String, name: String = "main chunk") {
		Dispatch.call(context, chunkLoader.loadTextChunk(Variable(env), name, code))
	}

	companion object {
		private val COUNTER = AtomicInteger()
	}
}
